@@ -19,7 +19,7 @@
 # 
 #----------------------------------------------------------------------------
 #
-# $Id: Context.pm,v 1.42 2000/02/01 12:14:28 abw Exp $
+# $Id: Context.pm,v 1.44 2000/02/29 18:12:24 abw Exp $
 #
 #============================================================================
 
@@ -35,7 +35,7 @@ use Template::Cache;
 use Template::Stash;
 
 
-$VERSION   = sprintf("%d.%02d", q$Revision: 1.42 $ =~ /(\d+)\.(\d+)/);
+$VERSION   = sprintf("%d.%02d", q$Revision: 1.44 $ =~ /(\d+)\.(\d+)/);
 $DEBUG     = 0;
 $CATCH_VAR = 'error';
 
@@ -78,7 +78,7 @@ sub new {
     my $params = shift || { };
     my ($cache, $stash, $pbase, $predefs) = 
 	@$params{ qw( CACHE STASH PLUGIN_BASE PRE_DEFINE ) };
-
+    my ($handler, $error);
 
     # stash is constructed with any PRE_DEFINE variables;
     # we add a 'global' namespace for convenience
@@ -112,8 +112,15 @@ sub new {
 	DEBUG        => $params->{ DEBUG }       || 0,
     }, $class;
 
-    $self->redirect(TEMPLATE_OUTPUT, $params->{ OUTPUT });
-    $self->redirect(TEMPLATE_ERROR,  $params->{ ERROR } || \*STDERR);
+    # install OUTPUT and ERROR handlers, using STDOUT/STDERR as default
+    ($handler, $error) = 
+	$self->redirect(TEMPLATE_OUTPUT, $params->{ OUTPUT });
+    ($handler, $error) = 
+	$self->redirect(TEMPLATE_ERROR,  $params->{ ERROR } || \*STDERR)
+	    unless $error;
+
+    # save any error generated by above redirection for callers to inspect
+    $self->{ _ERROR } = $error;
 
     $self;
 }
@@ -254,6 +261,12 @@ sub throw {
 	$info = $exception->info();
 	$exception->thrown(1);
     }
+    elsif (! $info) {
+	# if only one value was passed then the type is 'undef'
+	$info = $type;
+	$type = ERROR_UNDEF;
+    }
+#    print "type: $type   info: $info\n";
 
     # we first examine the exception type to see if a handler is defined.
     # if not, we strip off dotted elements from the end of the type to 
@@ -277,6 +290,7 @@ sub throw {
 
     # call sub-routine if $catch is a CODE ref
     if (($catchref = ref($catch)) eq 'CODE') {
+	print "throwing code handler\n";
 	$catch = &$catch($self, $type, $info);
     }
     # call process() to render a Template::Directive reference
@@ -292,7 +306,7 @@ sub throw {
     # create an exception if $catch is still undefined
     $catch = $catch || Template::Exception->new($type, $info, 1)
 	unless defined $catch;
-	
+
     return $catch;
 }
 
@@ -502,22 +516,23 @@ sub redirect {
     $where = $what eq Template::Constants::TEMPLATE_OUTPUT
 	     ? \*STDOUT
 	     : \*STDERR
-	unless defined $where;
+  	unless defined $where;
 
     # save previous handler
     $previous = $self->{ $what };
 
     # if $where is a string representing a filename, we prepend OUTPUT_PATH
     $where = $self->{ OUTPUT_PATH } . '/' . $where
-	unless ref($where);
+	unless ref($where) || $where =~ /^\//;
 
     # set handler internally
     ($self->{ $what }, $error) = (Template::Utils::output_handler($where));
-    $self->error($error)
-	if $error;
+
+#    $self->error($error)
+#	if $error;
 
     # return previous handler
-    $previous;
+    return wantarray ? ($previous, $error) : $previous;
 }
 
 
@@ -591,8 +606,8 @@ sub _evaluate {
     my ($self, $ops, $type) = @_;
     my ($root, $debug) = @$self{ qw( STASH DEBUG ) };
     my ($nops, $ip, $op);
-    my ($x, $y, $z, $p, $e, $val, $lflag, $err);
-    my (@stack, @pending, @expand);
+    my ($x, $y, $z, $p, $val, $lflag, $err);
+    my (@stack, @pending, @expand, @result);
     my $default_mode = 0;
 
     # return literal value intact
@@ -816,45 +831,36 @@ sub _evaluate {
 	    ($x, $y, $p) = splice(@stack, -3);    # x.y(p)
 	    $lflag = ($op == OP_LDOT);
 	    $p = [] unless $p;
-	    ($err, $e) = (); 
+	    @result = ();
+	    ($err, $z) = (); 
 
 	    # can't do anything if the LHS is undef/0 or the RHS is undefined.
 	    # the RHS may be 0 (e.g. mylist.0)
 	    push(@stack, undef), next
 		unless $x && defined $y;
 
-	    # setting $e will trigger an exception to be raised 
-	    # an exception or other error already in $err will take priority
-
 	    if ($y =~ /^[\._]/) {
-		($z, $e) = (undef, "invalid member name '$y'")
+		@result = (undef, "invalid member name '$y'")
 	    }
 	    elsif (UNIVERSAL::isa($x, 'Template::Stash')) {
-		($z, $err) = $x->get($y, $p, $self, $lflag);
-		unless (defined $z || $err) {
-		    # create an intermediate namespace hash if the item 
-		    # doesn't exist and this is an OP_LDOT (lvalue)
-		    if ($lflag) {
-			$err = $x->set($y, $z = { });
-		    }
+		@result = $x->get($y, $p, $lflag);
+		if (! defined $result[0] && $x eq $root) {
 		    # try to resolve undefined root variables
-		    elsif ($x eq $root) {
-			($z, $err) = &$z(@$p)
-			    if $z = $root_ops->{ $y };
+		    @result = &$z(@$p)
+			if $z = $root_ops->{ $y };
 		    }
-		}
 	    }
 	    elsif (ref($x) eq 'HASH') {
 		if (defined($z = $x->{ $y })) {
-		    ($z, $err) = &$z(@$p)   # execute any code binding
+		    @result = &$z(@$p)   # execute any code binding
 			if $z && ref($z) eq 'CODE';
 		}
 		elsif ($lflag) {
 		    # create empty hash if OP_LDOT
-		    $z = $x->{ $y } = { } if $lflag;
+		    @result = $x->{ $y } = { } if $lflag;
 		}
 		else {
-		    ($z, $err) = &$z($x, @$p) 
+		    @result = &$z($x, @$p) 
 			if $z = $hash_ops->{ $y };
 		}
 	    }
@@ -863,36 +869,46 @@ sub _evaluate {
 		# if the target is a list we try to apply the operations
 		# in $list_ops or apply a numerical operation as an index
 		if ($z) {
-		    ($z, $err) = &$z($x, @$p);
+		    @result = &$z($x, @$p);
 		}
 		else {
-		    $z = $x->[$y];
+		    @result = $x->[$y];
 		}
 	    }
 	    elsif (ref($x)) {
-		eval { ($z, $err) = $x->$y(@$p); };
+		eval { @result = $x->$y(@$p); };
 
 		# last chance to lookup a hash if the object call failed
 		if ($@ && UNIVERSAL::isa($x, 'HASH') 
 		       && defined($z = $x->{ $y })) {
-		    ($z, $err) = &$z(@$p)   # execute any code binding
+		    @result = &$z(@$p)   # execute any code binding
 			if $z && ref($z) eq 'CODE';
 		}
 		else {
-		    $e = $@ if $@;
+		    @result = (undef, $@) if $@;
 		}
 	    }
 	    else {
-		$e = "don't know how to access [ $x ].$y";
+		@result = (undef, "don't know how to access [ $x ].$y");
 	    }
 
-	    $e = "$y is undefined"	# NOTE $err may be defined and 0
-		if $debug && ! defined $z && ! $e && ! defined $err;
+	    # check results returned by above and fold to list ref if
+	    # multiple values are returned.  Undefined element 0 indicates
+	    # a failure and error is present is element 1
+	    if (@result) {
+		if (defined $result[0]) {
+		    $z = scalar @result > 1 ? [ @result ] : $result[0];
+		}
+		else {
+		    ($z, $err) = @result;
+		    $err = "$y is undefined" if $debug && ! defined $err;
+		}
+	    }
 
-	    # throw an exception into $err if $e is defined unless $err
-	    # is already set
-	    $err = $self->throw(ERROR_UNDEF, $e) 
-		if $e && ! $err;
+	    # throw any error we've collected and return if the error wasn't 
+	    # handled
+	    $err = $self->throw($err) 
+		if $err && $err !~ /^\d+$/;
 	    return (undef, $err) if $err;
 
 	    push(@stack, $z);
@@ -900,37 +916,45 @@ sub _evaluate {
 	}
 	elsif ($op == OP_LSET) {
 	    ($z, $x, $y, $p) = splice(@stack, -4);   # x.y(p) = z
+	    $err = '';
+	    @result = ();
 
 	    push(@stack, undef), next
 		unless $x && defined $y;
 
 	    if ($y =~ /^[\._]/) {
-		($z, $e) = (undef, "invalid member name '$y'")
+		($z, $err) = (undef, "invalid member name '$y'")
 	    }
 	    elsif (UNIVERSAL::isa($x, 'Template::Stash')) {
-		$err = $x->set($y, $z, $self)
+		$err = $x->set($y, $z)
 		    unless ($default_mode 
-			    && (($val, $err) = $x->get($y, $p, $self))
-			    && $val);
+			    && ($val = $x->get($y, $p)));
 	    }
 	    elsif (ref($x) eq 'HASH') {
 		$x->{ $y } = $z
 		    unless $default_mode && $x->{ $y };
 	    }
 	    elsif (ref($x)) {
-		eval { ($z, $err) = $x->$y($z)
+		eval { @result = $x->$y($z)
 			   unless ($default_mode 
-				   && (($val, $err) = $x->$y())
-				   && $val);
+				   && ($val = $x->$y()));
 		};
-		$e = $@ if $@;
+		if ($@) {
+		    $err = $@;
+		}
+		elsif (defined $result[0]) {
+		    $z = scalar @result > 1 ? [ @result ] : $result[0];
+		}
+		else {
+		    ($z, $err) = @result;
+		}
 	    }
 	    else {
-		$e = "don't know how to assign to [ $x ].$y";
+		$err = "don't know how to assign to [ $x ].$y";
 	    }
 
-	    $err = $self->throw(ERROR_UNDEF, $e) 
-		if $e && ! $err;
+	    $err = $self->throw(ERROR_UNDEF, $err) 
+		if $err && $err !~ /^\d+$/;
 	    return (undef, $err) if $err;
 
 	    push(@stack, $z);
@@ -1069,7 +1093,7 @@ Andy Wardley E<lt>cre.canon.co.ukE<gt>
 
 =head1 REVISION
 
-$Revision: 1.42 $
+$Revision: 1.44 $
 
 =head1 COPYRIGHT
 
